@@ -18,6 +18,8 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "config.h"
@@ -49,7 +51,7 @@
  * [Plugin]
  * Module=helloworld
  * Depends=foo;bar;baz
- * Loader=python3
+ * Loader=python
  * Name=Hello World
  * Description=Displays "Hello World"
  * Authors=Steve Frécinaux &lt;code@istique.net&gt;
@@ -62,48 +64,211 @@
 
 G_DEFINE_QUARK (peas-plugin-info-error, peas_plugin_info_error)
 
-G_DEFINE_BOXED_TYPE (PeasPluginInfo, peas_plugin_info,
-                     _peas_plugin_info_ref,
-                     _peas_plugin_info_unref)
+G_DEFINE_FINAL_TYPE (PeasPluginInfo, peas_plugin_info, G_TYPE_OBJECT)
 
-PeasPluginInfo *
-_peas_plugin_info_ref (PeasPluginInfo *info)
+enum {
+  PROP_0,
+  PROP_AUTHORS,
+  PROP_BUILTIN,
+  PROP_COPYRIGHT,
+  PROP_DEPENDENCIES,
+  PROP_DESCRIPTION,
+  PROP_HELP_URI,
+  PROP_HIDDEN,
+  PROP_ICON_NAME,
+  PROP_LOADED,
+  PROP_MODULE_DIR,
+  PROP_MODULE_NAME,
+  PROP_NAME,
+  PROP_VERSION,
+  PROP_WEBSITE,
+  N_PROPS
+};
+
+static GParamSpec *properties [N_PROPS];
+
+static void
+peas_plugin_info_finalize (GObject *object)
 {
-  g_atomic_int_inc (&info->refcount);
-  return info;
+  PeasPluginInfo *info = PEAS_PLUGIN_INFO (object);
+
+  g_clear_pointer (&info->filename, g_free);
+  g_clear_pointer (&info->module_dir, g_free);
+  g_clear_pointer (&info->data_dir, g_free);
+  g_clear_pointer (&info->embedded, g_free);
+  g_clear_pointer (&info->module_name, g_free);
+  g_clear_pointer (&info->dependencies, g_strfreev);
+  g_clear_pointer (&info->name, g_free);
+  g_clear_pointer (&info->desc, g_free);
+  g_clear_pointer (&info->icon_name, g_free);
+  g_clear_pointer (&info->website, g_free);
+  g_clear_pointer (&info->copyright, g_free);
+  g_clear_pointer (&info->version, g_free);
+  g_clear_pointer (&info->help_uri, g_free);
+  g_clear_pointer (&info->authors, g_strfreev);
+  g_clear_pointer (&info->schema_source, g_settings_schema_source_unref);
+  g_clear_pointer (&info->external_data, g_hash_table_unref);
+  g_clear_pointer (&info->error, g_error_free);
+  g_clear_pointer (&info->resources, g_ptr_array_unref);
+
+  G_OBJECT_CLASS (peas_plugin_info_parent_class)->finalize (object);
 }
 
-void
-_peas_plugin_info_unref (PeasPluginInfo *info)
+static void
+peas_plugin_info_get_property (GObject    *object,
+                               guint       prop_id,
+                               GValue     *value,
+                               GParamSpec *pspec)
 {
-  if (!g_atomic_int_dec_and_test (&info->refcount))
-    return;
+  PeasPluginInfo *info = PEAS_PLUGIN_INFO (object);
 
-  g_free (info->filename);
-  g_free (info->module_dir);
-  g_free (info->data_dir);
-  g_free (info->embedded);
-  g_free (info->module_name);
-  g_strfreev (info->dependencies);
-  g_free (info->name);
-  g_free (info->desc);
-  g_free (info->icon_name);
-  g_free (info->website);
-  g_free (info->copyright);
-  g_free (info->version);
-  g_free (info->help_uri);
-  g_strfreev (info->authors);
+  switch (prop_id)
+    {
+    case PROP_AUTHORS:
+      g_value_set_boxed (value, peas_plugin_info_get_authors (info));
+      break;
 
-  if (info->schema_source != NULL)
-    g_settings_schema_source_unref (info->schema_source);
+    case PROP_BUILTIN:
+      g_value_set_boolean (value, peas_plugin_info_is_builtin (info));
+      break;
 
-  if (info->external_data != NULL)
-    g_hash_table_unref (info->external_data);
+    case PROP_COPYRIGHT:
+      g_value_set_string (value, peas_plugin_info_get_copyright (info));
+      break;
 
-  if (info->error != NULL)
-    g_error_free (info->error);
+    case PROP_DEPENDENCIES:
+      g_value_set_boxed (value, peas_plugin_info_get_dependencies (info));
+      break;
 
-  g_free (info);
+    case PROP_DESCRIPTION:
+      g_value_set_string (value, peas_plugin_info_get_description (info));
+      break;
+
+    case PROP_HELP_URI:
+      g_value_set_string (value, peas_plugin_info_get_help_uri (info));
+      break;
+
+    case PROP_HIDDEN:
+      g_value_set_boolean (value, peas_plugin_info_is_hidden (info));
+      break;
+
+    case PROP_ICON_NAME:
+      g_value_set_string (value, peas_plugin_info_get_icon_name (info));
+      break;
+
+    case PROP_LOADED:
+      g_value_set_boolean (value, peas_plugin_info_is_loaded (info));
+      break;
+
+    case PROP_MODULE_DIR:
+      g_value_set_string (value, peas_plugin_info_get_module_dir (info));
+      break;
+
+    case PROP_MODULE_NAME:
+      g_value_set_string (value, peas_plugin_info_get_module_name (info));
+      break;
+
+    case PROP_NAME:
+      g_value_set_string (value, peas_plugin_info_get_name (info));
+      break;
+
+    case PROP_VERSION:
+      g_value_set_string (value, peas_plugin_info_get_version (info));
+      break;
+
+    case PROP_WEBSITE:
+      g_value_set_string (value, peas_plugin_info_get_website (info));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    }
+}
+
+static void
+peas_plugin_info_class_init (PeasPluginInfoClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->finalize = peas_plugin_info_finalize;
+  object_class->get_property = peas_plugin_info_get_property;
+
+  properties[PROP_BUILTIN] =
+    g_param_spec_boolean (I_("builtin"), NULL, NULL,
+                          FALSE,
+                          (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_LOADED] =
+    g_param_spec_boolean (I_("loaded"), NULL, NULL,
+                          FALSE,
+                          (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_MODULE_NAME] =
+    g_param_spec_string (I_("module-name"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_MODULE_DIR] =
+    g_param_spec_string (I_("module-dir"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_NAME] =
+    g_param_spec_string (I_("name"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_COPYRIGHT] =
+    g_param_spec_string (I_("copyright"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_DESCRIPTION] =
+    g_param_spec_string (I_("description"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_ICON_NAME] =
+    g_param_spec_string (I_("icon-name"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_WEBSITE] =
+    g_param_spec_string (I_("website"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_VERSION] =
+    g_param_spec_string (I_("version"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_HIDDEN] =
+    g_param_spec_boolean (I_("hidden"), NULL, NULL,
+                          FALSE,
+                          (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_HELP_URI] =
+    g_param_spec_string (I_("help-uri"), NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_DEPENDENCIES] =
+    g_param_spec_boxed (I_("dependencies"), NULL, NULL,
+                        G_TYPE_STRV,
+                        (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_AUTHORS] =
+    g_param_spec_boxed (I_("authors"), NULL, NULL,
+                        G_TYPE_STRV,
+                        (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
+}
+
+static void
+peas_plugin_info_init (PeasPluginInfo *info)
+{
 }
 
 /*
@@ -117,14 +282,14 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
  * Returns: a newly created #PeasPluginInfo.
  */
 PeasPluginInfo *
-_peas_plugin_info_new (const gchar *filename,
-                       const gchar *module_dir,
-                       const gchar *data_dir)
+_peas_plugin_info_new (const char *filename,
+                       const char *module_dir,
+                       const char *data_dir)
 {
   gsize i;
   gboolean is_resource;
-  gchar *loader = NULL;
-  gchar **strv, **keys;
+  char *loader = NULL;
+  char **strv, **keys;
   PeasPluginInfo *info;
   GKeyFile *plugin_file;
   GBytes *bytes = NULL;
@@ -134,11 +299,10 @@ _peas_plugin_info_new (const gchar *filename,
 
   is_resource = g_str_has_prefix (filename, "resource://");
 
-  info = g_new0 (PeasPluginInfo, 1);
-  info->refcount = 1;
+  info = g_object_new (PEAS_TYPE_PLUGIN_INFO, NULL);
 
   plugin_file = g_key_file_new ();
-  
+
   if (is_resource)
     {
       bytes = g_resources_lookup_data (filename + strlen ("resource://"),
@@ -147,7 +311,7 @@ _peas_plugin_info_new (const gchar *filename,
     }
   else
     {
-      gchar *content;
+      char *content;
       gsize length;
 
       if (g_file_get_contents (filename, &content, &length, &error))
@@ -223,7 +387,8 @@ _peas_plugin_info_new (const gchar *filename,
           goto error;
         }
     }
-  else if (is_resource)
+  else if (is_resource &&
+           info->loader_id != PEAS_UTILS_GJS_LOADER_ID)
     {
       g_warning ("Bad plugin file '%s': resource plugins must be embedded",
                  filename);
@@ -235,7 +400,7 @@ _peas_plugin_info_new (const gchar *filename,
                                                    "Plugin",
                                                    "Depends", NULL, NULL);
   if (info->dependencies == NULL)
-    info->dependencies = g_new0 (gchar *, 1);
+    info->dependencies = g_new0 (char *, 1);
 
   /* Get Description */
   info->desc = g_key_file_get_locale_string (plugin_file, "Plugin",
@@ -249,7 +414,7 @@ _peas_plugin_info_new (const gchar *filename,
   info->authors = g_key_file_get_string_list (plugin_file, "Plugin",
                                               "Authors", NULL, NULL);
   if (info->authors == NULL)
-    info->authors = g_new0 (gchar *, 1);
+    info->authors = g_new0 (char *, 1);
 
   /* Get Copyright */
   strv = g_key_file_get_string_list (plugin_file, "Plugin",
@@ -321,11 +486,8 @@ _peas_plugin_info_new (const gchar *filename,
 
 error:
 
-  g_free (info->embedded);
+  g_object_unref (info);
   g_free (loader);
-  g_free (info->module_name);
-  g_free (info->name);
-  g_free (info);
   g_clear_pointer (&bytes, g_bytes_unref);
   g_key_file_free (plugin_file);
 
@@ -442,7 +604,7 @@ peas_plugin_info_is_hidden (const PeasPluginInfo *info)
  *
  * Returns: the module name.
  */
-const gchar *
+const char *
 peas_plugin_info_get_module_name (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -462,7 +624,7 @@ peas_plugin_info_get_module_name (const PeasPluginInfo *info)
  *
  * Returns: the module directory.
  */
-const gchar *
+const char *
 peas_plugin_info_get_module_dir (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -483,7 +645,7 @@ peas_plugin_info_get_module_dir (const PeasPluginInfo *info)
  *
  * Returns: the plugin's data dir.
  */
-const gchar *
+const char *
 peas_plugin_info_get_data_dir (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -506,7 +668,7 @@ peas_plugin_info_get_data_dir (const PeasPluginInfo *info)
  */
 GSettings *
 peas_plugin_info_get_settings (const PeasPluginInfo *info,
-                               const gchar          *schema_id)
+                               const char           *schema_id)
 {
   GSettingsSchema *schema;
   GSettings *settings;
@@ -525,14 +687,14 @@ peas_plugin_info_get_settings (const PeasPluginInfo *info,
 
       if (!g_file_query_exists (gschema_compiled, NULL))
         {
-          const gchar *argv[] = {
+          const char *argv[] = {
             "glib-compile-schemas",
             "--targetdir", info->module_dir,
             info->module_dir,
             NULL
           };
 
-          g_spawn_sync (NULL, (gchar **) argv, NULL, G_SPAWN_SEARCH_PATH,
+          g_spawn_sync (NULL, (char **) argv, NULL, G_SPAWN_SEARCH_PATH,
                         NULL, NULL, NULL, NULL, NULL, NULL);
         }
 
@@ -581,12 +743,12 @@ peas_plugin_info_get_settings (const PeasPluginInfo *info,
  *
  * Returns: (transfer none): the plugin's dependencies.
  */
-const gchar **
+const char * const *
 peas_plugin_info_get_dependencies (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
 
-  return (const gchar **) info->dependencies;
+  return (const char * const *) info->dependencies;
 }
 
 /**
@@ -600,7 +762,7 @@ peas_plugin_info_get_dependencies (const PeasPluginInfo *info)
  */
 gboolean
 peas_plugin_info_has_dependency (const PeasPluginInfo *info,
-                                 const gchar          *module_name)
+                                 const char           *module_name)
 {
   guint i;
 
@@ -629,7 +791,7 @@ peas_plugin_info_has_dependency (const PeasPluginInfo *info,
  *
  * Returns: the plugin's name.
  */
-const gchar *
+const char *
 peas_plugin_info_get_name (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -650,7 +812,7 @@ peas_plugin_info_get_name (const PeasPluginInfo *info)
  *
  * Returns: the plugin's description.
  */
-const gchar *
+const char *
 peas_plugin_info_get_description (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -671,7 +833,7 @@ peas_plugin_info_get_description (const PeasPluginInfo *info)
  *
  * Returns: the plugin's icon name.
  */
-const gchar *
+const char *
 peas_plugin_info_get_icon_name (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -692,12 +854,12 @@ peas_plugin_info_get_icon_name (const PeasPluginInfo *info)
  *
  * Returns: (transfer none) (array zero-terminated=1): the plugin's author list.
  */
-const gchar **
+const char * const *
 peas_plugin_info_get_authors (const PeasPluginInfo *info)
 {
-  g_return_val_if_fail (info != NULL, (const gchar **) NULL);
+  g_return_val_if_fail (info != NULL, (const char **) NULL);
 
-  return (const gchar **) info->authors;
+  return (const char * const *) info->authors;
 }
 
 /**
@@ -710,7 +872,7 @@ peas_plugin_info_get_authors (const PeasPluginInfo *info)
  *
  * Returns: the plugin's associated website.
  */
-const gchar *
+const char *
 peas_plugin_info_get_website (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -728,7 +890,7 @@ peas_plugin_info_get_website (const PeasPluginInfo *info)
  *
  * Returns: the plugin's copyright information.
  */
-const gchar *
+const char *
 peas_plugin_info_get_copyright (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -746,7 +908,7 @@ peas_plugin_info_get_copyright (const PeasPluginInfo *info)
  *
  * Returns: the plugin's version.
  */
-const gchar *
+const char *
 peas_plugin_info_get_version (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -770,7 +932,7 @@ peas_plugin_info_get_version (const PeasPluginInfo *info)
  *
  * Returns: the plugin's help URI.
  */
-const gchar *
+const char *
 peas_plugin_info_get_help_uri (const PeasPluginInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
@@ -796,9 +958,9 @@ peas_plugin_info_get_help_uri (const PeasPluginInfo *info)
  *
  * Since: 1.6
  */
-const gchar *
+const char *
 peas_plugin_info_get_external_data (const PeasPluginInfo *info,
-                                    const gchar          *key)
+                                    const char           *key)
 {
   g_return_val_if_fail (info != NULL, NULL);
   g_return_val_if_fail (key != NULL, NULL);
@@ -811,3 +973,81 @@ peas_plugin_info_get_external_data (const PeasPluginInfo *info,
 
   return g_hash_table_lookup (info->external_data, key);
 }
+
+ /**
+ * peas_plugin_info_get_resource:
+ * @info: A #PeasPluginInfo.
+ * @filename: (allow-none): The filename of the resource, or %NULL.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Creates a new #GResource for the given @filename
+ * located in the module directory. If @filename is %NULL
+ * then "${module_name}.gresource" will be loaded.
+ *
+ * Returns: (transfer full): a new #GResource, or %NULL.
+ */
+GResource *
+peas_plugin_info_get_resource (const PeasPluginInfo  *info,
+                               const char           *filename,
+                               GError               **error)
+{
+  char *default_filename = NULL;
+  char *full_filename;
+  GResource *resource;
+
+  g_return_val_if_fail (info != NULL, NULL);
+  g_return_val_if_fail (filename == NULL || filename[0] != '\0', NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  if (filename == NULL)
+    {
+      default_filename = g_strconcat (info->module_name, ".gresource", NULL);
+      filename = (const char *) default_filename;
+    }
+
+  full_filename = g_build_filename (info->module_dir, filename, NULL);
+  resource = g_resource_load (full_filename, error);
+
+  g_free (full_filename);
+  g_free (default_filename);
+
+  return resource;
+}
+
+/**
+ * peas_plugin_info_load_resource:
+ * @info: A #PeasPluginInfo.
+ * @filename: (allow-none): The filename of the resource, or %NULL.
+ * @error: a #GError for error reporting, or %NULL.
+ *
+ * Loads the resource using peas_plugin_info_get_resource() and
+ * registers it. The resource's lifetime will be automatically
+ * handled by @info.
+ *
+ * See peas_plugin_info_get_resource() for more information.
+ */
+void
+peas_plugin_info_load_resource (const PeasPluginInfo  *info,
+                                const char            *filename,
+                                GError               **error)
+{
+  GResource *resource;
+  PeasPluginInfo *info_ = (PeasPluginInfo *) info;
+
+  resource = peas_plugin_info_get_resource (info, filename, error);
+
+  if (resource == NULL)
+    return;
+
+  if (info_->resources == NULL)
+    {
+      info_->resources = g_ptr_array_new ();
+      g_ptr_array_set_free_func (info_->resources,
+                                 (GDestroyNotify) g_resources_unregister);
+    }
+
+  g_resources_register (resource);
+
+  g_ptr_array_add (info_->resources, resource);
+  g_resource_unref (resource);
+}
